@@ -546,29 +546,38 @@ bool Manager::CreateTableStream(RecordVal* fval)
 		}
 
 	Val *want_record = fval->Lookup("want_record", true);
+	auto destination_is_set = dst->Type()->IsSet();
 
 	if ( val )
 		{
-		const BroType* table_yield = dst->Type()->AsTableType()->YieldType();
-		const BroType* compare_type = val;
-
-		if ( want_record->InternalInt() == 0 )
-			compare_type = val->FieldType(0);
-
-		if ( ! same_type(table_yield, compare_type) )
+		if ( destination_is_set )
 			{
-			ODesc desc1;
-			ODesc desc2;
-			compare_type->Describe(&desc1);
-			table_yield->Describe(&desc2);
-			reporter->Error("Input stream %s: Table type does not match value type. Need type '%s', got '%s'", stream_name.c_str(),
-					desc1.Description(), desc2.Description());
+			reporter->Error("Input stream %s: 'destination' field is a set, "
+			                "but the 'val' field was also specified "
+			                "(did you mean to use a table instead of a set?)",
+			                stream_name.data());
 			return false;
 			}
+		else
+			{
+			const BroType* table_yield = dst->Type()->AsTableType()->YieldType();
+			const BroType* compare_type = want_record->InternalInt() == 0 ? val->FieldType(0) : val;
+
+			if ( ! same_type(table_yield, compare_type) )
+				{
+				ODesc desc1;
+				ODesc desc2;
+				compare_type->Describe(&desc1);
+				table_yield->Describe(&desc2);
+				reporter->Error("Input stream %s: Table type does not match value type. Need type '%s', got '%s'",
+				                stream_name.c_str(), desc1.Description(), desc2.Description());
+				return false;
+				}
+			}
 		}
 	else
 		{
-		if ( ! dst->Type()->IsSet() )
+		if ( ! destination_is_set )
 			{
 			reporter->Error("Input stream %s: 'destination' field is a table,"
 			                " but 'val' field is not provided"
@@ -593,10 +602,12 @@ bool Manager::CreateTableStream(RecordVal* fval)
 			}
 
 		const type_list* args = etype->ArgTypes()->Types();
+		auto required_arg_count = destination_is_set ? 3 : 4;
 
-		if ( args->length() != 4 )
+		if ( args->length() != required_arg_count )
 			{
-			reporter->Error("Input stream %s: Table event must take 4 arguments", stream_name.c_str());
+			reporter->Error("Input stream %s: Table event must take %d arguments",
+			                stream_name.c_str(), required_arg_count);
 			return false;
 			}
 
@@ -623,26 +634,33 @@ bool Manager::CreateTableStream(RecordVal* fval)
 			return false;
 			}
 
-		if ( want_record->InternalInt() == 1 && ! same_type((*args)[3], val) )
+		if ( ! destination_is_set )
 			{
-			ODesc desc1;
-			ODesc desc2;
-			val->Describe(&desc1);
-			(*args)[3]->Describe(&desc2);
-			reporter->Error("Input stream %s: Table event's value attributes do not match. Need '%s', got '%s'", stream_name.c_str(),
-					desc1.Description(), desc2.Description());
-			return false;
-			}
-		else if (  want_record->InternalInt() == 0
-		           && !same_type((*args)[3], val->FieldType(0) ) )
-			{
-			ODesc desc1;
-			ODesc desc2;
-			val->FieldType(0)->Describe(&desc1);
-			(*args)[3]->Describe(&desc2);
-			reporter->Error("Input stream %s: Table event's value attribute does not match. Need '%s', got '%s'", stream_name.c_str(),
-					desc1.Description(), desc2.Description());
-			return false;
+			if ( want_record->InternalInt() == 1 && val && ! same_type((*args)[3], val) )
+				{
+				ODesc desc1;
+				ODesc desc2;
+				val->Describe(&desc1);
+				(*args)[3]->Describe(&desc2);
+				reporter->Error("Input stream %s: Table event's value attributes do not match. Need '%s', got '%s'",
+				                stream_name.c_str(), desc1.Description(), desc2.Description());
+				return false;
+				}
+			else if ( want_record->InternalInt() == 0 &&
+			          val && !same_type((*args)[3], val->FieldType(0) ) )
+				{
+				ODesc desc1;
+				ODesc desc2;
+				val->FieldType(0)->Describe(&desc1);
+				(*args)[3]->Describe(&desc2);
+				reporter->Error("Input stream %s: Table event's value attribute does not match. Need '%s', got '%s'",
+				                stream_name.c_str(), desc1.Description(), desc2.Description());
+				return false;
+				}
+			else if ( ! val )
+				{
+				reporter->Error("Encountered a null value when creating a table stream");
+				}
 			}
 
 		assert(want_record->InternalInt() == 1 || want_record->InternalInt() == 0);
@@ -1067,14 +1085,33 @@ Val* Manager::ValueToIndexVal(const Stream* i, int num_fields, const RecordType
 		for ( int j = 0 ; j < type->NumFields(); j++ )
 			{
 			if ( type->FieldType(j)->Tag() == TYPE_RECORD )
-				l->Append(ValueToRecordVal(i, vals,
-				          type->FieldType(j)->AsRecordType(), &position, have_error));
+				{
+				auto rv = ValueToRecordVal(i, vals, type->FieldType(j)->AsRecordType(),
+				                           &position, have_error);
+
+				if ( have_error )
+					break;
+
+				l->Append(rv);
+				}
 			else
 				{
-				l->Append(ValueToVal(i, vals[position], type->FieldType(j), have_error));
+				auto v = ValueToVal(i, vals[position], type->FieldType(j), have_error);
+
+				if ( have_error )
+					break;
+
+				l->Append(v);
 				position++;
 				}
 			}
+
+		if ( have_error )
+			{
+			Unref(l);
+			return nullptr;
+			}
+
 		idxval = l;
 		}
 
@@ -1319,10 +1356,7 @@ int Manager::SendEntryTable(Stream* i, const Value* const *vals)
 			{
 			ev = BifType::Enum::Input::Event->GetVal(BifEnum::Input::EVENT_NEW);
 			if ( stream->num_val_fields == 0 )
-				{
-				Ref(stream->description);
 				SendEvent(stream->event, 3, stream->description->Ref(), ev, predidx);
-				}
 			else
 				SendEvent(stream->event, 4, stream->description->Ref(), ev, predidx, valval->Ref());
 			}
@@ -1409,10 +1443,12 @@ void Manager::EndCurrentSend(ReaderFrontend* reader)
 
 		if ( stream->event )
 			{
-			Ref(predidx);
-			Ref(val);
-			Ref(ev);
-			SendEvent(stream->event, 4, stream->description->Ref(), ev, predidx, val);
+			if ( stream->num_val_fields == 0 )
+				SendEvent(stream->event, 3, stream->description->Ref(), ev->Ref(),
+				          predidx->Ref());
+			else
+				SendEvent(stream->event, 4, stream->description->Ref(), ev->Ref(),
+				          predidx->Ref(), val->Ref());
 			}
 
 		if ( predidx )  // if we have a stream or an event...
@@ -1529,8 +1565,10 @@ int Manager::SendEventStreamEvent(Stream* i, EnumVal* type, const Value* const *
 
 	if ( stream->want_record )
 		{
-		RecordVal * r = ValueToRecordVal(i, vals, stream->fields, &position, convert_error);
-		out_vals.push_back(r);
+		RecordVal* r = ValueToRecordVal(i, vals, stream->fields, &position, convert_error);
+
+		if ( ! convert_error )
+			out_vals.push_back(r);
 		}
 
 	else
@@ -1550,6 +1588,9 @@ int Manager::SendEventStreamEvent(Stream* i, EnumVal* type, const Value* const *
 				position++;
 				}
 
+			if ( convert_error)
+				break;
+
 			out_vals.push_back(val);
 			}
 		}
@@ -1680,7 +1721,7 @@ int Manager::PutTable(Stream* i, const Value* const *vals)
 					{
 					ev = BifType::Enum::Input::Event->GetVal(BifEnum::Input::EVENT_NEW);
 					if ( stream->num_val_fields == 0 )
-						SendEvent(stream->event, 4, stream->description->Ref(),
+						SendEvent(stream->event, 3, stream->description->Ref(),
 								ev, predidx);
 					else
 						SendEvent(stream->event, 4, stream->description->Ref(),
@@ -1740,7 +1781,6 @@ bool Manager::Delete(ReaderFrontend* reader, Value* *vals)
 		TableStream* stream = (TableStream*) i;
 		bool convert_error = false;
 		Val* idxval = ValueToIndexVal(i, stream->num_idx_fields, stream->itype, vals, convert_error);
-		assert(idxval != 0);
 		readVals = stream->num_idx_fields + stream->num_val_fields;
 		bool streamresult = true;
 
@@ -1750,6 +1790,8 @@ bool Manager::Delete(ReaderFrontend* reader, Value* *vals)
 			return false;
 			}
 
+		assert(idxval != nullptr);
+
 		if ( stream->pred || stream->event )
 			{
 			Val *val = stream->tab->Lookup(idxval);
@@ -1781,11 +1823,12 @@ bool Manager::Delete(ReaderFrontend* reader, Value* *vals)
 			// only if stream = true -> no streaming
 			if ( streamresult && stream->event )
 				{
-				Ref(idxval);
 				assert(val != 0);
-				Ref(val);
 				EnumVal *ev = BifType::Enum::Input::Event->GetVal(BifEnum::Input::EVENT_REMOVED);
-				SendEvent(stream->event, 4, stream->description->Ref(), ev, idxval, val);
+				if ( stream->num_val_fields == 0 )
+					SendEvent(stream->event, 3, stream->description->Ref(), ev, idxval->Ref());
+				else
+					SendEvent(stream->event, 4, stream->description->Ref(), ev, idxval->Ref(), val->Ref());
 				}
 			}
 
@@ -2006,7 +2049,12 @@ RecordVal* Manager::ValueToRecordVal(const Stream* stream, const Value* const *v
 			(*position)++;
 			}
 
-		if ( fieldVal )
+		if ( have_error )
+			{
+			Unref(rec);
+			return nullptr;
+			}
+		else if ( fieldVal )
 			rec->Assign(i, fieldVal);
 		}
 
@@ -2390,6 +2438,13 @@ Val* Manager::ValueToVal(const Stream* i, const Value* val, BroType* request_typ
 			{
 			Val* assignval = ValueToVal(i, val->val.set_val.vals[j], type, have_error);
 
+			if ( have_error )
+				{
+				Unref(t);
+				Unref(s);
+				return nullptr;
+				}
+
 			t->Assign(assignval, 0);
 			Unref(assignval); // index is not consumed by assign.
 			}
@@ -2406,7 +2461,16 @@ Val* Manager::ValueToVal(const Stream* i, const Value* val, BroType* request_typ
 		VectorVal* v = new VectorVal(vt);
 		for ( int j = 0; j < val->val.vector_val.size; j++ )
 			{
-			v->Assign(j, ValueToVal(i, val->val.vector_val.vals[j], type, have_error));
+			auto el = ValueToVal(i, val->val.vector_val.vals[j], type, have_error);
+
+			if ( have_error )
+				{
+				Unref(v);
+				Unref(vt);
+				return nullptr;
+				}
+
+			v->Assign(j, el);
 			}
 
 		Unref(vt);
@@ -2573,6 +2637,13 @@ Val* Manager::ValueToVal(const Stream* i, const Value* val, bool& have_error) co
 			{
 			Val* assignval = ValueToVal(i, val->val.set_val.vals[j], have_error);
 
+			if ( have_error )
+				{
+				Unref(t);
+				Unref(s);
+				return nullptr;
+				}
+
 			t->Assign(assignval, 0);
 			Unref(assignval); // index is not consumed by assign.
 			}
@@ -2600,7 +2671,16 @@ Val* Manager::ValueToVal(const Stream* i, const Value* val, bool& have_error) co
 		VectorVal* v = new VectorVal(vt);
 		for ( int j = 0; j < val->val.vector_val.size; j++ )
 			{
-			v->Assign(j, ValueToVal(i, val->val.vector_val.vals[j], have_error));
+			auto el = ValueToVal(i, val->val.vector_val.vals[j], have_error);
+
+			if ( have_error )
+				{
+				Unref(v);
+				Unref(vt);
+				return nullptr;
+				}
+
+			v->Assign(j, el);
 			}
 
 		Unref(vt);
